<?php
/**
 * Created by PhpStorm.
 * User: tsaricam
 * Date: 10/02/2023
 * Time: 15:06
 *
 * @since 1.13.0
 */

namespace WPCCrawler\Objects\Api\OpenAi\ShortCode;

use WPCCrawler\Objects\Api\OpenAi\OpenAiClient;
use WPCCrawler\Objects\ShortCode\AbstractCustomShortCodeApplier;
use WPCCrawler\Objects\ShortCode\ShortCodeApplier;

class OpenAiGptShortCodeApplier extends AbstractCustomShortCodeApplier {

    /** @var OpenAiClient */
    private $client;

    /**
     * @param OpenAiClient     $client The client that will be used to make requests to the API
     * @param ShortCodeApplier $applier See {@link AbstractCustomShortCodeApplier::__construct()}
     * @since 1.13.0
     */
    public function __construct(OpenAiClient $client, ShortCodeApplier $applier) {
        parent::__construct($applier);
        $this->client = $client;
    }

    public function getShortCodeName(): string {
        return OpenAiGptShortCode::getName();
    }

    /**
     * @param string $template The template that might contain one or more [openai-gpt] short codes
     * @return string The content whose [openai-gpt] short codes are replaced
     * @since 1.13.0
     */
    public function onApply(string $template): string {
        return $this->doApplyWithCallback($template, function(array $match) {
            return $this->applySingle($match);
        });
    }

    protected function onInjectValues(string $template): string {
        return $this->doApplyWithCallback($template, function(array $match) {
            return $this->injectValuesSingle($match);
        });
    }

    /**
     * @return OpenAiClient
     * @since 1.13.0
     */
    public function getClient(): OpenAiClient {
        return $this->client;
    }

    /*
     * HELPERS
     */

    /**
     * @param array $match Regular expression match array
     * @return string The short code's actual value
     * @since 1.13.0
     */
    protected function applySingle(array $match): string {
        $shortCode = $this->createShortCodeFromMatch($match);
        return $shortCode === null
            ? ''
            : $shortCode->apply();
    }

    /**
     * @param array $match Regular expression match array
     * @return string The short code rewritten by injecting other short codes via the applier retrieved via
     *                {@link getApplier()}.
     * @since 1.13.0
     */
    protected function injectValuesSingle(array $match): string {
        $shortCode = $this->createShortCodeFromMatch($match);
        return $shortCode === null
            ? ''
            : $shortCode->injectValues()->toString();
    }

    /**
     * @param array $match Regular expression match array
     * @return OpenAiGptShortCode|null The short code if it could be created. Otherwise, `null`.
     * @since 1.13.0
     */
    public function createShortCodeFromMatch(array $match): ?OpenAiGptShortCode {
        return OpenAiGptShortCode::fromMatch($this->getClient(), $this->getApplier(), $match);
    }

}