<?php
/**
 * Created by PhpStorm.
 * User: tsaricam
 * Date: 02/08/2024
 * Time: 08:30
 *
 * @since 1.15.0
 */

namespace WPCCrawler\Objects\Arrays;

use WPCCrawler\Utils;

class ArrayData {

    /** @var array */
    private $data;

    /**
     * @param array $data The array data
     * @since 1.15.0
     */
    public function __construct(array $data) {
        $this->data = $data;
    }

    /**
     * @param string $key See {@link getValue()}
     * @return array|null If the value exists, and it is an array, the value is returned. Otherwise, `null` is returned.
     * @since 1.15.0
     */
    public function getArray(string $key): ?array {
        $value = $this->getValue($key);
        return is_array($value)
            ? $value
            : null;
    }

    /**
     * @param string $key See {@link getScalar()}
     * @return float|int|string|null If the value exists, and it is a numeric value, the value is returned. Otherwise,
     *                               `null` is returned.
     * @since 1.15.0
     */
    public function getNumeric(string $key) {
        $value = $this->getScalar($key);
        return is_numeric($value)
            ? $value
            : null;
    }

    /**
     * @param string $key See {@link getScalar()}
     * @return string|null If the value exists, and it is a scalar, the value is returned as a string. Otherwise, `null`
     *                       is returned.
     * @since 1.15.0
     */
    public function getString(string $key): ?string {
        $value = $this->getScalar($key);
        return $value === null
            ? null
            : (string) $value;
    }

    /**
     * @param string $key  See {@link getValue()}
     * @return scalar|null If the value exists, and it is a scalar, the value is returned. Otherwise, `null` is
     *                     returned.
     * @since 1.15.0
     */
    public function getScalar(string $key) {
        $value = $this->getValue($key);
        return is_scalar($value)
            ? $value
            : null;
    }

    /*
     *
     */

    /**
     * @param string $dotKey A key whose value will be returned. This can be in dot notation as well.
     * @return mixed|null If the key has a value, that value is returned. Otherwise, `null` is returned.
     * @since 1.15.0
     */
    protected function getValue(string $dotKey) {
        return Utils::array_get($this->getData(), $dotKey);
    }

    /*
     *
     */

    /**
     * @return array The array data
     * @since 1.15.0
     */
    public function getData(): array {
        return $this->data;
    }

}