<?php
/**
 * Created by PhpStorm.
 * User: tsaricam
 * Date: 03/12/2023
 * Time: 09:07
 *
 * @since 1.15.0
 */

namespace WPCCrawler\Objects\Filtering\Commands\ActionCommands\Base;

use WPCCrawler\Objects\Crawling\Bot\AbstractBot;
use WPCCrawler\Objects\Enums\ValueType;
use WPCCrawler\Objects\Filtering\Interfaces\NeedsBot;
use WPCCrawler\WPCCrawler;

abstract class AbstractUpdateSiteSettingActionCommand extends AbstractActionCommand implements NeedsBot {

    /** @var AbstractBot|null */
    private $bot;

    public function getInputDataTypes(): array {
        return [ValueType::T_SETTING];
    }

    protected function isOutputTypeSameAsInputType(): bool {
        return true;
    }

    public function doesNeedSubjectValue(): bool {
        return false;
    }

    protected function isTestable(): bool {
        return false;
    }

    protected function onExecute($key, $subjectValue) {
        $logger = $this->getLogger();

        $bot = $this->getBot();
        if (!$bot) {
            return;
        }

        $siteId = $bot->getSiteId();
        if (!$siteId) {
            if ($logger) {
                $logger->addMessage(_wpcc('Site ID could not be retrieved.'));
            }

            return;
        }

        // If this is a test, do not update the setting
        if (WPCCrawler::isDoingGeneralTest() || WPCCrawler::isDoingUnitTest()) {
            if ($logger) {
                $logger->addMessage(_wpcc('The site setting is not updated, because this is a test.'));
            }

            return;
        }

        $this->onUpdateSiteSetting($siteId);
    }

    /**
     * @param int $siteId ID of the site whose setting should be updated
     * @since 1.15.0
     */
    abstract protected function onUpdateSiteSetting(int $siteId): void;

    /*
     *
     */

    public function setBot(?AbstractBot $bot): void {
        $this->bot = $bot;
    }

    public function getBot(): ?AbstractBot {
        return $this->bot;
    }

}