<?php
/**
 * Created by PhpStorm.
 * User: tsaricam
 * Date: 09/03/2024
 * Time: 19:23
 *
 * @since 1.15.0
 */

namespace WPCCrawler\Objects\Filtering\Commands\ActionCommands\Crawling;

use WPCCrawler\Objects\Filtering\Commands\ActionCommands\Base\AbstractCrawlingActionCommand;
use WPCCrawler\Objects\Filtering\Commands\Views\ViewDefinition;
use WPCCrawler\Objects\Filtering\Commands\Views\ViewDefinitionList;
use WPCCrawler\Objects\Filtering\Enums\CommandKey;
use WPCCrawler\Objects\Filtering\Enums\InputName;
use WPCCrawler\Objects\Views\Enums\ViewVariableName;
use WPCCrawler\Objects\Views\NumericInputWithLabel;

class Wait extends AbstractCrawlingActionCommand {

    public function getKey(): string {
        return CommandKey::WAIT;
    }

    public function getName(): string {
        return _wpcc('Wait');
    }

    public function getDescription(): ?string {
        return _wpcc('Pauses the execution for the specified amount of time');
    }

    protected function createViews(): ?ViewDefinitionList {
        return (new ViewDefinitionList())
            // Duration in seconds
            ->add((new ViewDefinition(NumericInputWithLabel::class))
                ->setVariable(ViewVariableName::TITLE, _wpcc('Duration (seconds)'))
                ->setVariable(ViewVariableName::INFO, _wpcc('Specify the duration for which the script'
                    . ' execution should wait, in seconds. For instance, you can enter 30 to wait for 30 seconds.'
                    . ' Decimal values are also accepted. For example, to wait for 100 milliseconds, you can enter 0.1.'
                ))
                ->setVariable(ViewVariableName::NAME, InputName::DURATION_SECONDS)
            )
        ;
    }

    protected function onExecute($key, $subjectValue) {
        $microseconds = $this->getDurationInMicroseconds();
        usleep($microseconds);
    }

    /*
     *
     */

    /**
     * @return int The wait duration in microseconds. This is always a non-negative value.
     * @since 1.15.0
     */
    public function getDurationInMicroseconds(): int {
        $seconds = $this->getFloatOption(InputName::DURATION_SECONDS);
        if ($seconds === null) {
            return 0;
        }

        $microseconds = (int) ($seconds * 1000 * 1000);
        return max(0, $microseconds);
    }

}