<?php
/**
 * Created by PhpStorm.
 * User: tsaricam
 * Date: 22/03/2024
 * Time: 15:53
 *
 * @since 1.15.0
 */

namespace WPCCrawler\Objects\Filtering\Commands\ActionCommands\Date;

use DateTime;
use WPCCrawler\Objects\Filtering\Commands\ActionCommands\Base\AbstractDateActionCommand;
use WPCCrawler\Objects\Filtering\Commands\Views\ViewDefinition;
use WPCCrawler\Objects\Filtering\Commands\Views\ViewDefinitionList;
use WPCCrawler\Objects\Filtering\Enums\CommandKey;
use WPCCrawler\Objects\Filtering\Enums\InputName;
use WPCCrawler\Objects\Views\Enums\ViewVariableName;
use WPCCrawler\Objects\Views\InputWithLabel;

class DateSetToFixedDate extends AbstractDateActionCommand {

    public function getKey(): string {
        return CommandKey::DATE_SET_TO_FIXED_DATE;
    }

    public function getName(): string {
        return _wpcc('Set to fixed date');
    }

    protected function createViews(): ?ViewDefinitionList {
        return (new ViewDefinitionList())
            ->add((new ViewDefinition(InputWithLabel::class))
                ->setVariable(ViewVariableName::TITLE, _wpcc('New date'))
                ->setVariable(ViewVariableName::INFO,  _wpcc('The date that will be assigned as the new value.')
                    . ' ' .  _wpcc_enter_date_with_format()
                )
                ->setVariable(ViewVariableName::NAME,  InputName::DATE)
                ->setVariable(ViewVariableName::TYPE,  'text'))
            ;
    }

    protected function onModifyDate(DateTime $dateTime): DateTime {
        $logger = $this->getLogger();

        $dateStr = $this->getDateOption();
        if ($dateStr === null) {
            if ($logger) $logger->addMessage(_wpcc('A fixed date value is not provided. The date is not changed.'));
            return $dateTime;
        }

        $newDate = $this->createDateTime($dateStr);
        if ($newDate === null) {
            if ($logger) $logger->addMessage(_wpcc('The given date value cannot be parsed. The date is not changed.'));
            return $dateTime;
        }

        return $newDate;
    }

    /*
     *
     */

    /**
     * @return string|null The fixed date string provided as an option to this command
     * @since 1.15.0
     */
    protected function getDateOption(): ?string {
        $date = $this->getStringOption(InputName::DATE);
        if (!is_string($date)) {
            return null;
        }

        $date = trim($date);
        return $date === ''
            ? null
            : $date;
    }
}