<?php
/**
 * Created by PhpStorm.
 * User: tsaricam
 * Date: 10/03/2024
 * Time: 09:32
 *
 * @since 1.15.0
 */

namespace WPCCrawler\Objects\Filtering\Commands\ActionCommands\Element;

use DOMElement;
use Symfony\Component\DomCrawler\Crawler;
use WPCCrawler\Objects\Filtering\Commands\ActionCommands\Base\AbstractBotActionCommand;
use WPCCrawler\Objects\Filtering\Enums\CommandKey;
use WPCCrawler\Objects\Html\DomNodeSimplifier;
use WPCCrawler\Utils;

class ElementRemoveDataAttributes extends AbstractBotActionCommand {

    public function getKey(): string {
        return CommandKey::ELEMENT_REMOVE_DATA_ATTRS;
    }

    public function getName(): string {
        return _wpcc('Remove data attributes');
    }

    public function getDescription(): ?string {
        return _wpcc('Removes all the attributes whose name starts with "data-" from the element and all the'
            . ' elements in it');
    }

    protected function onExecuteCommand($node): void {
        if (!($node instanceof Crawler)) return;

        $target = $node->getNode(0);
        if (!($target instanceof DOMElement)) return;

        // Remove the data attributes
        $simplifier = new DomNodeSimplifier($target);
        $simplifier->removeDataAttributes();

        $logger = $this->getLogger();
        if ($logger) {
            $logger->addModifiedSubjectItem(Utils::getDomNodeHtmlString($target));
        }
    }

}