<?php
/**
 * Created by PhpStorm.
 * User: tsaricam
 * Date: 11/03/2024
 * Time: 14:38
 *
 * @since 1.15.0
 */

namespace WPCCrawler\Objects\Filtering\Commands\ActionCommands\Text;

use WPCCrawler\Objects\Crawling\Bot\AbstractBot;
use WPCCrawler\Objects\Filtering\Commands\ActionCommands\Base\AbstractTextActionCommand;
use WPCCrawler\Objects\Filtering\Commands\CommandUtils;
use WPCCrawler\Objects\Filtering\Commands\Views\ViewDefinition;
use WPCCrawler\Objects\Filtering\Commands\Views\ViewDefinitionList;
use WPCCrawler\Objects\Filtering\Enums\CommandKey;
use WPCCrawler\Objects\Filtering\Enums\InputName;
use WPCCrawler\Objects\Filtering\Interfaces\NeedsBot;
use WPCCrawler\Objects\Views\Enums\ViewVariableName;
use WPCCrawler\Objects\Views\MultipleSelectorWithAttribute;

class TextAssign extends AbstractTextActionCommand implements NeedsBot {

    /**
     * @var string Name of the default attribute whose value will be retrieved from the found HTML elements as the new
     *      value
     */
    const DEFAULT_EL_ATTR = 'html';

    /** @var AbstractBot|null */
    private $bot;

    public function getKey(): string {
        return CommandKey::TEXT_ASSIGN;
    }

    public function getName(): string {
        return _wpcc('Assign');
    }

    protected function createViews(): ?ViewDefinitionList {
        return (new ViewDefinitionList())
            // Selectors and attributes, used to find the new value
            ->add((new ViewDefinition(MultipleSelectorWithAttribute::class))
                ->setVariable(ViewVariableName::TITLE, _wpcc('Value selectors'))
                ->setVariable(ViewVariableName::NAME,  InputName::CSS_SELECTOR)
                ->setVariable(ViewVariableName::INFO,  _wpcc('Enter one or more selectors that will be used to find the new value.'))
                ->setVariable(ViewVariableName::DEFAULT_ATTR, self::DEFAULT_EL_ATTR)
            )
        ;
    }

    protected function onModifyText(?string $text): ?string {
        $newValue = $this->findNewValue();
        return $newValue !== null
            ? $newValue
            : $text;
    }

    /*
     *
     */

    /**
     * @return string|null If it is found, the new value. Otherwise, `null`.
     * @since 1.15.0
     */
    protected function findNewValue(): ?string {
        $logger = $this->getLogger();

        $bot = $this->getBot();
        if (!$bot) {
            if ($logger) $logger->addMessage(_wpcc('Value could not be assigned, because there is no bot.'));
            return null;
        }

        $commandUtils = new CommandUtils();

        // Get the CSS selectors
        $selectors = $commandUtils->getCssSelectorsOption($this);
        if ($selectors === null) {
            if ($logger) $logger->addMessage(_wpcc('Value could not be assigned, because a CSS selector is not defined.'));
            return null;
        }

        $value = $bot->extractValuesWithMultipleSelectorData($bot->getCrawler(), $selectors, self::DEFAULT_EL_ATTR, false, true);
        if (!is_string($value)) {
            if ($logger) $logger->addMessage(_wpcc('Value could not be assigned, because a value cannot be found.'));
            return null;
        }

        return $value;
    }

    /*
     *
     */

    public function setBot(?AbstractBot $bot): void {
        $this->bot = $bot;
    }

    public function getBot(): ?AbstractBot {
        return $this->bot;
    }

}