<?php
/**
 * Created by PhpStorm.
 * User: tsaricam
 * Date: 20/07/2024
 * Time: 16:48
 *
 * @since 1.15.0
 */

namespace WPCCrawler\Objects\Filtering\Property\Error;

use WPCCrawler\Objects\Enums\ValueType;
use WPCCrawler\Objects\Error\Base\AbstractError;
use WPCCrawler\Objects\Filtering\Commands\Base\AbstractBaseCommand;
use WPCCrawler\Objects\Filtering\Enums\PropertyKey;
use WPCCrawler\Objects\Filtering\Property\Base\AbstractProperty;
use WPCCrawler\Objects\Filtering\Property\Objects\CalculationResult;

class ErrorTextProperty extends AbstractProperty {

    public function getKey(): string {
        return PropertyKey::ERROR_TEXT;
    }

    public function getName(): string {
        return _wpcc('Error text');
    }

    public function getDescription(): ?string {
        return _wpcc('This property includes the error message and all the details about the error, including 
            the details of its exception, if available.');
    }

    public function getInputDataTypes(): array {
        return [ValueType::T_ERROR];
    }

    public function getOutputDataTypes(): array {
        return [ValueType::T_STRING];
    }

    protected function onCalculate($key, $source, AbstractBaseCommand $cmd): ?CalculationResult {
        if (!($source instanceof AbstractError)) {
            return null;
        }

        $text = $source->getInfo()->__toString();
        return new CalculationResult($key, $text);
    }
}